@@ -24,7 +24,9 @@
     if (!is.null(object$pCCA)) 
         ev.account <- ev.account - summ$partial.chi
     summ$ev.con.account <- cumsum(summ$ev.con)/ev.account
-    summ$ev.uncon.account <- cumsum(summ$ev.uncon)/ev.account
+    summ$ev.uncon.account <-
+        (summ$constr.chi + cumsum(summ$ev.uncon))/ev.account
+    summ$cca.acc <- cumsum(summ$ev.con)/summ$constr.chi
     summ$ev.head <- c(summ$ev.con, summ$ev.uncon)[1:axes]
     summ$scaling <- scaling
     summ$digits <- digits
